var API = require('./api')

var RESERVED_NAMES = {
  FAAS_ENV: true
  , FAAS_API_KEY: true
  , FAAS_API_SECRET: true
}

////
// faas.env
//
// faas.env(...) loads configuration settings from faas.io for the
// credentials and environment name specified, and merges the name/value
// pairs returned with the local process environment variables. It makes
// the results available through faas.env[] and as an argument to the
// callback function provided.
//
// Usage:
//
// 1. Set external environment variables for FAAS_API_KEY, FAAS_API_SECRET,
//    and (optionally) FAAS_ENV (the name of the environment to load).
//    Access settings via faas.env[] (faas.env[] doesn't have to be used
//    directly in the callback scope, as it would be available anywhere
//    via require('faas').env[]).
//
// faas.env(function(){
//   // Access env settings in here via faas.env.SETTING
// })
//
// 2. Pass options to faas.env() instead of relying on
//    process.env.FAAS_API_KEY, etc.
//
// faas.env({ apiKey: XXX, apiSecret: XXX, env: XXX }, function(){...})
//
// The process.env[] variables FAAS_API_KEY, FAAS_API_SECRET, FAAS_ENV will
// be used to fill in missing options.
//
// 3. With either of the above methods, you can also request the faas.env
//    object as an argument to your callback. This could theoretically be
//    useful if you wanted to load multiple different environments
//    into your app, which would clash in the module namespace.
//
// faas.env(function(faasEnv){
//   // Use faasEnv.SETTING instead of faas.env.SETTING
// })
//
// faas.env({ apiKey: ... }, function(faasEnv){ ... })
//
////
var exportedEnv
exports.env = exportedEnv = function(options, cb_env){
  // Figure out if we have options or a callback.
  if ((typeof options === 'function') && !cb_env) {
    // Looks like the first argument was a callback.
    cb_env = options
    options = {}
  }

  options = options || {} // Provide a default object for ease of use.
  // Set up the query parameters for the API call.
  var query = {}
  if (options.apiKey) { query.apiKey = options.apiKey }
  if (options.apiSecret) { query.apiSecret = options.apiSecret }
  var envName = options.env || process.env.FAAS_ENV
  if (envName) { query.env = envName }

  var apiOpts = { url: { pathname: '/config/getSettings', query: query } }

  API.api(apiOpts, function(err, faasEnv){
    if (err) { error_cb(err); return }

    mergeProcessEnv(faasEnv)
    mergeApiCredentials(faasEnv, options)
    storeAtTopLevel(faasEnv)

    if (cb_env) { cb_env(faasEnv) }
  })

}

var mergeProcessEnv = function(faasEnv){
  // Apply the process environment to this object.
  for (var p in process.env) { if (process.env.hasOwnProperty(p)) {
    faasEnv[p] = process.env[p]
    // Alias FAAS_XXX to XXX.
    if (p.slice(0,5) === 'FAAS_' && !RESERVED_NAMES[p]) {
      faasEnv[p.slice(5)] = process.env[p]
    }
  }}
}

var mergeApiCredentials = function(faasEnv, options){
  // Add the API_KEY and API_SECRET to the env.
  if (options.apiKey) { faasEnv.FAAS_API_KEY = options.apiKey }
  if (options.apiSecret) { faasEnv.FAAS_API_SECRET = options.apiSecret }
}

var storeAtTopLevel = function(faasEnv){
  for (var p in faasEnv) { if (faasEnv.hasOwnProperty(p)) {
    exportedEnv[p] = faasEnv[p]
  }}
}

var error_cb = function(err){
  console.log('ERROR LOADING FAAS ENV:', err)
  console.log('Exiting due to error.')
  process.exit(1)
}


exportedEnv.detailsInfo = function(){
  var info = {}
  var dets = exportedEnv.FAAS_DETAILS
  for (var key in dets) { if (dets.hasOwnProperty(key)) {
    info[key] = [ exportedEnv[key], dets[key] ]
  }}
  return info
}
